﻿using SDON.Serialization;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;

namespace SDON
{
    /// <summary>
    /// Object that contains a reference to a DataTable defined in the Diagram object. Used to fill out the schema defined in the DataTable array in the Diagram object.
    /// </summary>
    [Serializable]
    public sealed class DataTableInstance : SDONSerializeable
    {
        [IgnoreIfDefaultValue(Default = null)]
        [DataMember(Name = "TableName")]
        private string _tableName = null;

        [IgnoreIfDefaultValue(Default = null)]
        [DataMember(Name = "Rows")]
        private List<DataRow> _rows = null;

        /// <summary>
        /// The name of a predefined table in the template used.
        /// </summary>
        public string TableName
        {
            get { return _tableName; }
            set { _tableName = value; }
        }

        /// <summary>
        /// The rows to insert in the pre-existing table.
        /// </summary>
        public List<DataRow> Rows
        {
            get { return _rows; }
            set { _rows = value; }
        }

        /// <summary>
        /// Add a row to a DataTableInstance
        /// </summary>
        /// <returns></returns>
        public DataRow AddDataRow()
        {
            if (Rows == null)
                Rows = new List<DataRow>();

            int id = Rows.Count() + 1;

            return AddDataRow(id.ToString());
        }

        /// <summary>
        /// Add a row to a DataTableInstance
        /// </summary>
        /// <param name="id">rowID</param>
        /// <returns></returns>
        public DataRow AddDataRow(string id)
        {
            if (Rows == null)
                Rows = new List<DataRow>();

            // If there's already a row with this ID... just return it
            foreach (DataRow row in Rows) {
                if (row.RowID == id)
                    return row;
            }

            DataRow dataRow = new DataRow();
            dataRow.RowID = id;

            Rows.Add(dataRow);

            return dataRow;
        }

        /// <summary>
        /// Default constructor.
        /// </summary>
        public DataTableInstance()
        {
        }

        /// <summary>
        /// Special constructor for implementing ISerializeable. Deserializes the object.
        /// </summary>
        /// <param name="info">Serialization info (all the values that are on the incoming object graph) of the object being deserialized.</param>
        /// <param name="context">The deserialization context.</param>
        internal DataTableInstance(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

    }
}
