﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using SDON.Serialization;

namespace SDON
{
    /// <summary>
    /// Object that represents the title of a diagram.
    /// </summary>
    [Serializable]
    public sealed class Title : SDONSerializeable, TextInterfaceCore<Title>, TextInterfaceExtended<Title>
    {
        [DataMember(Name = "Label")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _label = null;

        [DataMember(Name = "TextBold")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _textBold = null;

        [DataMember(Name = "TextItalic")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _textItalic = null;

        [DataMember(Name = "TextUnderline")]
        [IgnoreIfDefaultValue(Default = null)]
        private bool? _textUnderline = null;

        [DataMember(Name = "TextSize")]
        [IgnoreIfDefaultValue(Default = null)]
        private double _textSize = -1;

        [DataMember(Name = "TextFont")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _textFont = null;

        [DataMember(Name = "TextColor")]
        [IgnoreIfDefaultValue(Default = null)]
        private string _textColor = null;

        [DataMember(Name = "Truncate")]
        [IgnoreIfDefaultValue(Default = -2)]
        private int _truncate = -2;

        [DataMember(Name = "TextHyperlink")]
        [IgnoreIfDefaultValue(Default = null)]
        private Hyperlink _textHyperlink = null;

        /// <summary>
        /// Set the label value
        /// </summary>
        /// <param name="label">Label text</param>
        /// <returns>Title</returns>
        public Title SetLabel(string label)
        {
            _label = label;

            return this;
        }


        /// <summary>
        /// Bold value for label    
        /// </summary>
        /// <param name="textBold">Bold text</param>
        /// <returns>Title</returns>
        public Title SetTextBold(bool textBold)
        {
            _textBold = textBold;

            return this;
        }

        /// <summary>
        /// Italic text on label   
        /// </summary>
        /// <param name="textItalic">Italic text</param>
        /// <returns>Title</returns>
        public Title SetTextItalic(bool textItalic)
        {
            _textItalic = textItalic;

            return this;
        }

        /// <summary>
        /// Underlined text in label   
        /// </summary>
        /// <param name="textUnderline">Underlined text</param>
        /// <returns>Title</returns>
        public Title SetTextUnderline(bool textUnderline)
        {
            _textUnderline = textUnderline;

            return this;
        }

        /// <summary>
        /// Point size of text.  
        /// </summary>
        /// <param name="textSize">Size of text</param>
        /// <returns>Title</returns>
        public Title SetTextSize(double textSize)
        {
            _textSize = textSize;

            return this;
        }

        /// <summary>
        /// Set the font            
        /// </summary>
        /// <param name="textFont">Font name</param>
        /// <returns>Title</returns>
        public Title SetTextFont(string textFont)
        {
            _textFont = textFont;

            return this;
        }

        /// <summary>
        /// Set the text color of the label.   
        /// </summary>
        /// <param name="textColor">Text color</param>
        /// <returns>Title</returns>
        public Title SetTextColor(string textColor)
        {
            _textColor = textColor;

            return this;
        }

        /// <summary>
        /// Set a hyperlink for the text label   
        /// </summary>
        /// <param name="textHyperlink">Hyperlink URL</param>
        /// <returns>Title</returns>
        public Title SetTextHyperlink(string textHyperlink)
        {
            Hyperlink hl = new Hyperlink();

            hl.url = textHyperlink;

            _textHyperlink = hl;

            return this;
        }

        /// <summary>
        /// Defines the number of characters to allow in a shape before the remaining text is truncated. By default nothing is truncated. Defining Truncate to “-1” turns it off if on by default.
        /// </summary>
        /// <param name="truncate">Number of characters before truncation</param>
        /// <returns>Title</returns>
        public Title SetTextTruncate(int truncate)
        {
            _truncate = truncate;

            return this;
        }

        /// <summary>
        /// Default constructor.
        /// </summary>
        public Title()
        {
        }

        /// <summary>
        /// Special constructor for implementing ISerializeable. Deserializes the object.
        /// </summary>
        /// <param name="info">Serialization info (all the values that are on the incoming object graph) of the object being deserialized.</param>
        /// <param name="context">The deserialization context.</param>
        internal Title(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }
    }
}
