﻿// Copyright (c) 2019 SmartDraw, LLC. All rights reserved.
// www.visualscript.com/licenses/sdk-license.htm
// Version: 0.1.0

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Text;
using System.Threading.Tasks;
using System.Web.Script.Serialization;
using SDON;

namespace SDON
{
    /// <summary>
    /// Utility class for bulding SDON.
    /// </summary>
    public class SDONBuilder
    {
        /// <summary>
        /// Turns the SDON Diagram into a JSON string.
        /// </summary>
        /// <param name="diagram">The diagram to turn into JSON.</param>
        /// <returns>json string</returns>
        public static string ToJSON(DocumentBase diagram)
        {
            return ToJSONGeneric<DocumentBase>(ref diagram);
        }

        /// <summary>
        ///
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="diagram">Document</param>
        /// <returns>json string</returns>
        public static string ToJSONGeneric<T>(ref T diagram)
        {
            if (diagram == null) return null;

            DataContractJsonSerializerSettings settings = new DataContractJsonSerializerSettings();
            settings.EmitTypeInformation = EmitTypeInformation.Never;

            DataContractJsonSerializer serializer = new DataContractJsonSerializer(typeof(T), settings);
            MemoryStream stream = new MemoryStream();

            serializer.WriteObject(stream, diagram);
            stream.Seek(0, SeekOrigin.Begin);

            string json = System.Text.Encoding.UTF8.GetString(stream.ToArray());
            return json;
        }



        /// <summary>
        /// Turns a JSON string into a Document object.
        /// </summary>
        /// <param name="json">The SDON JSON.</param>
        /// <returns></returns>
        public static Document FromJSON(string json)
        {
            if (json == null) return null;

            if (json.Contains("\"__type\":\"Shape:#SDON\"") == true) //if there is type information in the JSON we must use the DataContract serializer. If not, we have to use the JavaScript serializer.
            {
                var serializer = new DataContractJsonSerializer(typeof(Document));
                MemoryStream stream = new MemoryStream(System.Text.Encoding.UTF8.GetBytes(json));

                Document diagram = (Document)serializer.ReadObject(stream);

                return diagram;
            }
            else
            {
                JavaScriptSerializer serializer = new JavaScriptSerializer();
                Document diagram = serializer.Deserialize<Document>(json);

                return diagram;
            }
        }
    }
}
